local _, addon = ...
local util = addon.utility

local _G = _G
local core = addon.core

local function addToSet(set, key, value)
   set[key] = value
end

local function removeFromSet(set, key)
   set[key] = nil
end

function util.setContains(set, key)
   return set[key] ~= nil
end

function util.getSetValue(set, key)
   return set[key]
end

local ClassColors = {}
addToSet(ClassColors,"DEATHKNIGHT",0xC41F3B)
addToSet(ClassColors,"DEMONHUNTER",0xA330C9)
addToSet(ClassColors,"DRUID"      ,0xFF7D0A)
addToSet(ClassColors,"HUNTER"     ,0xABD473)
addToSet(ClassColors,"MAGE"       ,0x69CCF0)
addToSet(ClassColors,"MONK"       ,0x00FF96)
addToSet(ClassColors,"PALADIN"    ,0xF58CBA)
addToSet(ClassColors,"PRIEST"     ,0xFFFFFF)
addToSet(ClassColors,"ROGUE"      ,0xFFF569)
addToSet(ClassColors,"SHAMAN"     ,0x0070DE)
addToSet(ClassColors,"WARLOCK"    ,0x9482C9)
addToSet(ClassColors,"WARRIOR"    ,0xC79C6E)
addToSet(ClassColors,"EMPHASIS"   ,0x00C800)

util.TimeList = { "AllTime", "Year", "Month", "Week", "Day", "Session" }

util.TimeSet = {}
for i = 1, #util.TimeList do
   addToSet(util.TimeSet, util.TimeList[i], util.TimeList[i] ~= "AllTime")
end

local SECOND = 1
local MINUTE = 60 * SECOND
local HOUR = 60 * MINUTE
local DAY = 24 * HOUR
local WEEK = 7 * DAY

local GoldIcon    = "|TInterface\\MoneyFrame\\UI-GoldIcon:12:12|t"
local SilverIcon  = "|TInterface\\MoneyFrame\\UI-SilverIcon:12:12|t"
local CopperIcon  = "|TInterface\\MoneyFrame\\UI-CopperIcon:12:12|t"

local LDBinteraction = false

local SexyCompatDefault = true

local DefaultFont = "Friz"
local DefaultFontSize = 12

local DaysToAdvance = 0

local realm = GetRealmName()

local DOWDefault = 3

function util.GetDOWSetting()
   local dowSetting = _G.WOWMMChar.DOWSetting or DOWDefault
   return dowSetting
end

function util.SetDOWSetting(dow)
   if GetDOWSetting() ~= dow then
      _G.WOWMMChar.DOWSetting = dow
   end
   cm:Refresh()
end

function util.AdvanceDays(days)
   DaysToAdvance = DaysToAdvance + days
end

function util.GetDaysToAdvance()
   return DaysToAdvance
end

function util.ResetDays()
   DaysToAdvance = 0
end

function util.UpdateEarnedSpent(cash, pcash)
   if pcash ~= nil then
      if cash >= pcash then
         local earned = cash - pcash
         _G.WOWMMGlobal[realm].AllTime.Earned = _G.WOWMMGlobal[realm].AllTime.Earned + earned
         _G.WOWMMGlobal[realm].Year.Earned = _G.WOWMMGlobal[realm].Year.Earned + earned
         _G.WOWMMGlobal[realm].Month.Earned = _G.WOWMMGlobal[realm].Month.Earned + earned
         _G.WOWMMGlobal[realm].Week.Earned = _G.WOWMMGlobal[realm].Week.Earned + earned
         _G.WOWMMGlobal[realm].Day.Earned = _G.WOWMMGlobal[realm].Day.Earned + earned
         _G.WOWMMGlobal[realm].Session.Earned = _G.WOWMMGlobal[realm].Session.Earned + earned
      else
         local spent = pcash - cash
         _G.WOWMMGlobal[realm].AllTime.Spent = _G.WOWMMGlobal[realm].AllTime.Spent + spent
         _G.WOWMMGlobal[realm].Year.Spent = _G.WOWMMGlobal[realm].Year.Spent + spent
         _G.WOWMMGlobal[realm].Month.Spent = _G.WOWMMGlobal[realm].Month.Spent + spent
         _G.WOWMMGlobal[realm].Week.Spent = _G.WOWMMGlobal[realm].Week.Spent + spent
         _G.WOWMMGlobal[realm].Day.Spent = _G.WOWMMGlobal[realm].Day.Spent + spent
         _G.WOWMMGlobal[realm].Session.Spent = _G.WOWMMGlobal[realm].Session.Spent + spent
      end
   end
end

function util.SetLDBInteraction(interaction)
   LDBinteraction = interaction
end

function util.IsLDBInteraction()
   return LDBinteraction
end

function util.GetClassRGB(class)
   if not class then
      return
   end
   local colors = {}
   local color = ClassColors[class]
   
   if not color then
      return
   end

   for i=1,3 do
      table.insert(colors, 1, bit.band(color, 0x0000FF)/0xFF)
      color = bit.rshift(color, 8)
   end
   return unpack(colors)
end

function util.GetSortedCharList(server)
   local chars = {}
   for n, v in pairs(_G.WOWMMGlobal) do
      if n == server and type(v) == "table" then
         for n, _ in pairs(v.Chars) do
            table.insert(chars, n)
         end
         table.sort(chars)
      end
   end
   return chars
end

function util.FormatMoney(cash)
	local value = abs(cash)
	local gold = floor(value / 10000)
	local silver = floor(mod(value / 100, 100))
	local copper = floor(mod(value, 100))

	return format("%d%s %02d%s %02d%s", gold, GoldIcon, silver, SilverIcon, copper, CopperIcon)
end

function util.FormatCurrency(currency, icon)
   return format("|T%s:12:12|t %s", icon, currency)
end

function util.GetRealmTotalCash(Realm)
   local total = 0
   for name, value in pairs(_G.WOWMMGlobal[Realm].Chars) do
   	total = total + value.Cash
   end
   return total
end

local function GetDate()
   local date = C_Calendar.GetDate()
   return date.weekday, date.month, date.monthDay, date.year
end

local function GetCalendarMonthInfo(o)
   local monthInfo = C_Calendar.GetMonthInfo(o)
   return monthInfo.month, monthInfo.year, monthInfo.numDays, monthInfo.firstWeekday
end

local function GetCalendarMonthOffset()
   local _, currentMonth, _, currentYear = GetDate()
   local calMonth, calYear = GetCalendarMonthInfo(0)
   local offset = 0

   if calYear ~= currentYear then
      local fullYearCount = (currentYear - calYear) - 1
      local monthOff = currentMonth + (12 - calMonth)
      offset = (fullYearCount * 12) + monthOff
   elseif calMonth ~= currentMonth then
      offset = currentMonth - calMonth
   end

   return offset
end

function util.GetCalendarDate(daysToAdvance)
   local weekday, month, day, year = GetDate()

   local monthOffset = GetCalendarMonthOffset()
   local _, _, monthDays = GetCalendarMonthInfo(monthOffset)

   while (day + daysToAdvance) > monthDays do
      local daysToNextMonth = (monthDays - day) + 1
      daysToAdvance = daysToAdvance - daysToNextMonth
      monthOffset = monthOffset + 1
      month, year, monthDays, weekday = GetCalendarMonthInfo(monthOffset)
      day = 1
   end
   day = day + daysToAdvance
   weekday = (((weekday - 1) + (daysToAdvance%7))%7) + 1

   return weekday, month, day, year, monthDays
end

function util.GetTimeSinceEpoch(daysToAdvance)
   if daysToAdvance == nil then return nil end
   return (time() + (daysToAdvance * DAY))
end

function util.GetFontSetting()
   local font = _G.WOWMMChar.Font
   if font == nil then
      font = DefaultFont
   end
   return font
end

function util.SetFontSetting(font)
   if util.GetFontSetting() ~= font then
      _G.WOWMMChar.Font = font
   end
end

function util.GetFontSizeSetting()
   local fontSize = _G.WOWMMChar.FontSize
   if fontSize == nil then
      fontSize = DefaultFontSize
   end
   return fontSize
end

function util.SetFontSizeSetting(fontSize)
   if util.GetFontSizeSetting() ~= fontSize then
      _G.WOWMMChar.FontSize = fontSize
   end
end

function util.GetSexyCompatSetting()
   local isSexyCompat = _G.WOWMMChar.Minimap.IsSexyCompat
   if isSexyCompat == nil then
      isSexyCompat = SexyCompatDefault
   end
   return isSexyCompat
end

function util.RegisterEvents(self, handler, ...)
	for i = 1, select('#', ...) do
      self:RegisterEvent(select(i, ...))
	end
   self:SetScript("OnEvent", handler)
end

function util.ResetAvg(key)
   if util.setContains(util.TimeSet, key) then
      local net = key.."Net"
      local count = key.."Count"
      _G.WOWMMGlobal[realm][net] = 0
      _G.WOWMMGlobal[realm][count] = 0
   end
end

function util.ResetTime(key)
   if util.setContains(util.TimeSet, key) then
      _G.WOWMMGlobal[realm][key].Earned = 0
      _G.WOWMMGlobal[realm][key].Spent = 0
   end
end
      
function util.ResetAll()
   for k, _ in pairs(util.TimeSet) do
      util.ResetTime(k)
      util.ResetAvg(k)
   end
end

function util.RemoveCharacter(Realm, CharName)
   _G.WOWMMGlobal[Realm].Chars[CharName] = nil
   _G.WOWMMGlobal[Realm].Classes[CharName] = nil

   local count = 0
   for _ in pairs(_G.WOWMMGlobal[Realm].Chars) do count = count + 1 end
   if count == 0 then
      if Realm ~= realm then
         _G.WOWMMGlobal[Realm] = nil
      else
         core.InitializePersistantVariables(false)
      end
   else
      _G.WOWMMGlobal[Realm].Total = util.GetRealmTotalCash(Realm)
   end
end
